#include <LiquidCrystal_I2C.h>
LiquidCrystal_I2C lcd(0x27, 16, 2);

#define BUZZER PD2 // pin 2
#define LED PB5 // pin 13
#define MIC PC0 // analog pin 0

int tonePin = 2;
int frequency = 1000;
int micPin = 0;
int dotLength = 240; // 240 miliseconds is the basic dot length

// Other lengths are computed from dot length
int dotSpace = dotLength;
int dashLength = dotLength*3;
int letterSpace = dotLength*3;
int wordSpace = dotLength*7;

// WPM = 1200/dotLength.
// For other speeds, use dotLength = 1200/(WPM)
float wpm = 1200./dotLength; // 240 gives 5 words per minute (WPM) speed.
  
int t1, t2, onTime, gap;
bool newLetter = false, newWord = false;
bool keyboardText = false, letterFound;
bool soundSequence = false;
int count = 0;
int lineLength = 0;
int maxLineLength = 16;
int soundThreshold = 300;
int cursorLine = 0;

char* letters[] = 
{
".-", "-...", "-.-.", "-..", ".", "..-.", "--.", "....", "..", // A-I
".---", "-.-", ".-..", "--", "-.", "---", ".--.", "--.-", ".-.", // J-R 
"...", "-", "..-", "...-", ".--", "-..-", "-.--", "--.." // S-Z
};

char* numbers[] = 
{
"-----", ".----", "..---", "...--", "....-", //0-4
".....", "-....", "--...", "---..", "----." //5-9
};

String dashSeq = "";
char keyLetter, ch;
int i;

void setup() 
{
  DDRB |= (1 << LED);
  DDRB |= (1 << BUZZER);
  DDRD &= ~(1 << MIC);
  lcd.init();
  lcd.backlight();
  Serial.begin(9600);
  lcd.home();
  lcd.print("Morse Code");
  lcd.setCursor(3, 1);
  lcd.print("coder/decoder");
  delay(1000);
  lcd.clear();
  lcd.home();
  lcd.print("Speed=");
  lcd.print(wpm);
  lcd.print("wpm");
  lcd.setCursor(7, 1);
  lcd.print("dot=");
  lcd.print(dotLength);
  lcd.print("ms");
  delay(1000);
  lcd.clear();
  lcd.home();
  lcd.print("Enter text or");
  lcd.setCursor(2, 1);
  lcd.print("sound sequence");
}

void loop()
{
  
  if (Serial.available() > 0)
  {
    if (keyboardText == false) {
      soundSequence = false;
    }
    
    keyboardText = true;
    ch = Serial.read();
    if (ch >= 'a' && ch <= 'z')
    { ch = ch-32; }
    
    if (ch >= 'A' && ch <= 'Z')
    {
      lcd.clear();
      lcd.home();
      lcd.print(ch);
      lcd.print(" ");
      lcd.print(letters[ch-'A']);
      flashSequence(letters[ch-'A']);
      delay(letterSpace);
    }
    if (ch >= '0' && ch <= '9')
    {
      lcd.clear();
      lcd.home();
      lcd.print(ch);
      lcd.print(" ");
      lcd.print(numbers[ch-'0']);
      flashSequence(numbers[ch-'0']);
      delay(letterSpace);
    }
    if (ch == ' ')
    { delay(wordSpace); } 
  
     if (Serial.available() <= 0) 
     {
      lcd.clear();
      lcd.home();
      lcd.print("Enter text or");
      lcd.setCursor(0, 1);
      lcd.print("  sound sequence");
      keyboardText = false;
     }
  }

  if (analogRead(micPin) > soundThreshold)
  {
    if (soundSequence == false) {
      soundSequence = true;
      lcd.clear();
    }
    count = 0;
    
    newLetter = true; 
    newWord = true;
    t1 = millis();
  
    PORTB |= (1 << LED);
    while (count < 12)
    {
      if(analogRead(micPin) > soundThreshold) count = 0;
      count++;
    }
  
    t2 = millis();
    onTime = t2-t1; // length of dot or dash keyed in
    PORTB &= ~(1 << LED);
    
    if (onTime <= dotLength*1.5) // allow for 50% longer 
      {dashSeq = dashSeq + ".";} // build dot/dash sequence
    else 
      {dashSeq = dashSeq + "-";}
  }

  gap=millis()-t2;
  if (newLetter == true && gap >= (letterSpace * 0.75)) // assume 
  {
    letterFound = false; keyLetter = 63;
    for (i=0; i<=25; i++)
    {
      if (dashSeq == letters[i]) // check if it's a letter
      {
        keyLetter = i+65;
        letterFound = true;   
        break; // letter found, stop searching
      }
    }

    if(letterFound == false) // check if it's a number
    {
      for (i=0; i<=10; i++)
      {
      if (dashSeq == numbers[i]) 
        {
          keyLetter = i+48;
          letterFound = true;   
          break; // number found, stop searching
        }
      }
    }    
    lcd.print(keyLetter);
    newLetter = false;
    dashSeq = "";
    lineLength = lineLength + 1;
  }  

  if ((newWord == true) && gap >= (wordSpace *0.75))
    { 
    newWord = false;
    lcd.print("_");
    lineLength = lineLength + 1;

    PORTB |= (1 << LED);
    delay(25);
    PORTB &= ~(1 << LED);
    }

  if (lineLength >= maxLineLength) 
    {
      if (cursorLine == 0) { 
        cursorLine = 1;
      } 
      else { 
        cursorLine = 0;
        delay(1000);
        lcd.clear();
      }
      lcd.setCursor(0, cursorLine);
      lineLength = 0;
    }      
} 

void flashSequence(char* sequence)
{
  int i = 0;
  while (sequence[i] == '.' || sequence[i] == '-')
  {
    flashDotOrDash(sequence[i]);
    i++;
  }
}

void flashDotOrDash(char dotOrDash)
{
  PORTB |= (1 << LED);
  tone(tonePin, frequency);
  if (dotOrDash == '.')
   { delay(dotLength); }
     else
   { delay(dashLength); }

  PORTB &= ~(1 << LED);
  noTone(tonePin);
  delay(dotLength); 
}
